@@ -1,20 +1,21 @@
       subroutine sbart(penalt,dofoff,xs,ys,ws,n,knot,nk,coef,sz,lev,
-     &crit,icrit,spar,ispar,lspar,uspar,tol,isetup,xwy,hs0,hs1,hs2,hs3,
-     &sg0,sg1,sg2,sg3,abd,p1ip,p2ip,ld4,ldnk,ier)
+     & crit,icrit,spar,ispar,lspar,uspar,tol,isetup,xwy,
+     & hs0,hs1,hs2,hs3, sg0,sg1,sg2,sg3,abd,p1ip,p2ip,ld4,ldnk,ier)
       implicit double precision(a-h,o-z)
       integer n,nk,isetup,icrit,ispar,ld4,ldnk,ier
-      double precision penalt,dofoff,xs(n),ys(n),ws(n),knot(nk+4),coef(
-     &nk),sz(n),lev(n),crit,spar,lspar,uspar,tol,xwy(nk),hs0(nk),hs1(nk)
-     &,hs2(nk),hs3(nk), sg0(nk),sg1(nk),sg2(nk),sg3(nk),abd(ld4,nk),
-     &p1ip(ld4,nk),p2ip(ldnk,nk)
-      double precisiont1,t2,ratio, a,b,c,d,e,eps,xm,p,q,r,tol1,tol2,u,v,
-     &w, fu,fv,fw,fx,x,ax,bx
+      double precision penalt,dofoff,xs(n),ys(n),ws(n),knot(nk+4),
+     &	coef(nk),sz(n),lev(n),crit,spar,lspar,uspar,tol,xwy(nk),hs0(nk),
+     &	hs1(nk),hs2(nk),hs3(nk), sg0(nk),sg1(nk),sg2(nk),sg3(nk),
+     &	abd(ld4,nk),p1ip(ld4,nk),p2ip(ldnk,nk)
+      double precision t1,t2,ratio, a,b,c,d,e,eps,xm,p,q,r,tol1,tol2,
+     &	u,v,w, fu,fv,fw,fx,x,ax,bx
       common /XXXsbart/q
       integer i
+
       i=1
 23000 if(.not.(i.le.n))goto 23002
       if(.not.(ws(i).gt.0))goto 23003
-      ws(i)=sqrt(ws(i)) 
+      ws(i)=sqrt(ws(i))
 23003 continue
       i=i+1
       goto 23000
@@ -22,16 +23,16 @@
       if(.not.(isetup.eq.0))goto 23005
       call sgram(sg0,sg1,sg2,sg3,knot,nk)
       call stxwx(xs,ys,ws,n,knot,nk,xwy,hs0,hs1,hs2,hs3)
-      t1=0. 
+      t1=0.
       t2=0.
-      do 23007 i=3,nk-3 
-      t1 = t1 + hs0(i) 
+      do 23007 i=3,nk-3
+      t1 = t1 + hs0(i)
 23007 continue
-      do 23009 i=3,nk-3 
-      t2 = t2 + sg0(i) 
+      do 23009 i=3,nk-3
+      t2 = t2 + sg0(i)
 23009 continue
       ratio = t1/t2
-      isetup = 1 
+      isetup = 1
 23005 continue
       if(.not.(ispar.eq.1))goto 23011
       call sslvrg(penalt,dofoff,xs,ys,ws,n,knot,nk,coef,sz,lev,crit,
@@ -39,7 +40,7 @@
      &p2ip,ld4,ldnk,ier)
       return
 23011 continue
-      ax=lspar 
+      ax=lspar
       bx=uspar
       c = 0.5*(3. - sqrt(5e0))
 c      eps = 1e0
@@ -124,9 +125,9 @@ c  update  a, b, v, w, and x
 c
       if (fu .gt. fx) go to 60
       if(u .ge. x) then
-         a = x
+	 a = x
       else
-         b = x
+	 b = x
       endif
 23043 continue
       v = w
@@ -137,9 +138,9 @@ c
       fx = fu
       go to 20
 60    if(u .lt. x) then
-         a = u
+	 a = u
       else
-         b = u
+	 b = u
       endif
       if (fu .le. fw) go to 70
       if (w .eq. x) go to 70
@@ -156,8 +157,8 @@ c
 80    v = u
       fv = fu
       go to 20
-90    continue 
-      spar = x 
+90    continue
+      spar = x
       crit = fx
       return
 23012 continue
