@@ -365,7 +365,7 @@ SEXP NewName(SEXP base, SEXP tag, int i, int n, int seqno)
     base = EnsureString(base);
     tag = EnsureString(tag);
     if (*CHAR(base) && *CHAR(tag)) {
-	ans = allocString(strlen(CHAR(base)) + strlen(CHAR(tag)));
+	ans = allocString(strlen(CHAR(base)) + strlen(CHAR(tag)) + 1);
 	sprintf(CHAR(ans), "%s.%s", CHAR(base), CHAR(tag));
     }
     else if (*CHAR(base)) {
