@@ -27,7 +27,6 @@
  */
 
 #include "Mathlib.h"
-#include "Error.h"
 
 double pwilcox(double x, double m, double n) {
   int i;
@@ -48,11 +47,11 @@ double pwilcox(double x, double m, double n) {
     return ML_NAN;
   }
   if (m >= WILCOX_MMAX) {
-    warning("m should be less than %d\n", WILCOX_MMAX);
+    MATHLIB_WARNING("m should be less than %d\n", WILCOX_MMAX);
     return ML_NAN;
   }
   if (n >= WILCOX_NMAX) {
-    warning("n should be less than %d\n", WILCOX_NMAX);
+    MATHLIB_WARNING("n should be less than %d\n", WILCOX_NMAX);
     return ML_NAN;
   }
   x = floor(x + 0.5);
